// $Id$
/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.httpinvoker;

import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.Before;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UsernameNotFoundException;

import eu.etaxonomy.cdm.api.application.CdmApplicationState;
import eu.etaxonomy.cdm.api.service.IUserService;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.taxeditor.remoting.cache.CdmTransientEntityCacher;
import eu.etaxonomy.taxeditor.session.ICdmEntitySession;
import eu.etaxonomy.taxeditor.session.MockSessionOwner;

/**
 * @author cmathew
 * @date 12 Jun 2015
 *
 */
public abstract class RemotingSessionAwareTest extends BaseRemotingTest {

    private static final Logger logger = Logger.getLogger(RemotingSessionAwareTest.class);

    protected CdmTransientEntityCacher cacher;
    protected MockSessionOwner sessionOwner;

    private static String extraUsername = "Someone";
    private static String extraPassword = "Password";


    private IUserService userService = getRemoteApplicationController().getUserService();

    @Before
    public void initializeSession() {

        sessionOwner = new MockSessionOwner();
        logger.info("created mock session owner : " + sessionOwner);
        ICdmEntitySession cdmEntitySession = cdmEntitySessionManager.newSession(sessionOwner, true);
        CdmApplicationState.getCurrentDataChangeService().register(sessionOwner);
        sessionOwner.setCdmEntitySession(cdmEntitySession);
        cacher = getCacher(sessionOwner);

        UserDetails extraUser = null;

        try {
            extraUser = userService.loadUserByUsername(extraUsername);
        } catch (UsernameNotFoundException unfe) {
            User user = User.NewInstance(extraUsername, extraPassword);
            userService.createUser(user);
        }


    }

    @After
    public void postSessionAwareTest() {
        sessionOwner.dispose();
        CdmApplicationState.getCurrentDataChangeService().unregister(sessionOwner);
        logger.info("disposed of mock session owner : " + sessionOwner);
    }


    protected static void authenticateExtraUser() {
        authenticate(extraUsername, extraPassword);
    }

}
