// $Id$
/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.httpinvoker;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.URL;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Platform;
import org.junit.BeforeClass;
import org.osgi.framework.Bundle;
import org.unitils.UnitilsJUnit4;

/**
 * @author cmathew
 * @date 11 Nov 2015
 *
 */
public abstract class TestConfig extends UnitilsJUnit4 {

    private static final Logger logger = Logger.getLogger(TestConfig.class);

    private final static String DEFAULT_USER = "admin";
    private final static String DEFAULT_PASSWORD = "00000";
    private final static int DEFAULT_HTTP_PORT = 9090;
    private final static String DEFAULT_CONTEXT_PATH = "";
    private final static String DEFAULT_SOURCE_NAME = "default";
    private final static String DEFAULT_HOST = "localhost";

    private static String userHomeKey = "user.home";

    protected static String user = DEFAULT_USER;
    protected static String password = DEFAULT_PASSWORD;
    protected static int httpPort = DEFAULT_HTTP_PORT;
    protected static String contextPath = DEFAULT_CONTEXT_PATH;
    protected static String sourceName = DEFAULT_SOURCE_NAME;
    protected static String host = DEFAULT_HOST;

    @BeforeClass
    public static void initializeTestConfig() {
        try {
            String userHomeDirPath;
            Bundle bundle = Platform.getBundle("eu.etaxonomy.taxeditor.test");

            URL userHomeDirURL = bundle.getEntry("src/test/resources");
            File userHomeDir = new File(FileLocator.resolve(userHomeDirURL).toURI());
            userHomeDirPath = userHomeDir.getAbsolutePath();

            URL serverPropertiesURL = bundle.getEntry("src/test/resources/server.properties");
            File serverPropertiesFile = new File(FileLocator.resolve(serverPropertiesURL).toURI());
            InputStream inputStream = new FileInputStream(serverPropertiesFile);

            Properties prop = new Properties();

            prop.load(inputStream);
            inputStream.close();


            logger.warn("Setting user.home to " + userHomeDirPath);
            System.setProperty(userHomeKey, userHomeDirPath);


            if(prop.getProperty("user") != null) {
                user = prop.getProperty("user");
            }

            if(prop.getProperty("password") != null) {
                password = prop.getProperty("password");
            }

            if(prop.getProperty("httpPort") != null) {
                httpPort = Integer.valueOf(prop.getProperty("httpPort"));
            }

            if(prop.getProperty("contextPath") != null) {
                contextPath = prop.getProperty("contextPath");
            }

        } catch (Exception e) {
            e.printStackTrace();
            // Assert.fail("Server failed to start. Reason : " + e.getMessage());
        }
    }

}
