// $Id$
/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.operation;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.eclipse.core.commands.ExecutionException;
import org.junit.Assert;
import org.junit.Ignore;
import org.junit.Test;
import org.unitils.dbunit.annotation.DataSet;

import eu.etaxonomy.cdm.api.service.IPolytomousKeyService;
import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.taxeditor.navigation.key.polytomous.operation.RemotingDeletePolytomousKeyOperation;
import eu.etaxonomy.taxeditor.navigation.key.polytomous.operation.RemotingUpdatePolytomousKeyAllNodesOperation;

/**
 * @author cmathew
 * @date 25 Jun 2015
 *
 */
@DataSet
public class PolytomousKeyViewPartTest extends BaseOperationTest {

    IPolytomousKeyService polytomousKeyService = getRemoteApplicationController().getPolytomousKeyService();
    private static UUID pkey1Uuid = UUID.fromString("f82cef88-5a9e-4917-9938-d08bda40836f");
    private static UUID pkey2Uuid = UUID.fromString("a00ca7ec-b660-433c-bd14-596826ab2243");
    private static UUID pkey3Uuid = UUID.fromString("197bc542-bf5b-4a52-aa66-c65df32c196b");

    @Ignore // waiting for #3998 to be fixed
    @Test
    public void testDeletePolytomousKey() throws ExecutionException {
        List<PolytomousKey> pKeysBeforeDelete = polytomousKeyService.list(PolytomousKey.class, null, null, null, null);
        int initialCount = pKeysBeforeDelete.size();

        List<PolytomousKey> keysToDelete = new ArrayList<PolytomousKey>();
        keysToDelete.add(polytomousKeyService.load(pkey1Uuid));
        keysToDelete.add(polytomousKeyService.load(pkey2Uuid));
        keysToDelete.add(polytomousKeyService.load(pkey3Uuid));

        List<UUID> uuids = new ArrayList<UUID>();
        uuids.add(pkey1Uuid);
        uuids.add(pkey2Uuid);
        uuids.add(pkey3Uuid);

        int keysToDeleteCount = keysToDelete.size();
        operation = new RemotingDeletePolytomousKeyOperation(sessionOwner,
                false,
                uuids);
        operation.execute(monitor, info);

        List<PolytomousKey> pKeysAfterDelete = polytomousKeyService.list(PolytomousKey.class, null, null, null, null);
        Assert.assertEquals(initialCount - keysToDeleteCount, pKeysAfterDelete.size());
        for(PolytomousKey key : keysToDelete) {
            Assert.assertFalse(pKeysAfterDelete.contains(key));
        }

    }

    @Test
    public void testUpdateAllNodes() throws ExecutionException {
        operation = new RemotingUpdatePolytomousKeyAllNodesOperation(sessionOwner,
                false,
                pkey1Uuid);
        operation.execute(monitor, info);
    }

}
