// $Id$
/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.operation;

import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.unitils.dbunit.annotation.DataSet;

import eu.etaxonomy.cdm.api.service.ITaxonNodeService;
import eu.etaxonomy.cdm.api.service.UpdateResult;
import eu.etaxonomy.cdm.api.service.UpdateResult.Status;
import eu.etaxonomy.cdm.api.service.dto.CdmEntityIdentifier;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.remoting.cache.ProxyUtils;
import eu.etaxonomy.taxeditor.store.operations.RemotingTestUpdateOperation;

/**
 * @author cmathew
 * @date 26 Jun 2015
 *
 */
@DataSet
public class RemotingCdmUpdateOperationTest extends BaseOperationTest {

    @SuppressWarnings("unused")
	private static final Logger logger = Logger.getLogger(RemotingCdmUpdateOperationTest.class);

    private static ITaxonNodeService taxonNodeService = getRemoteApplicationController().getTaxonNodeService();

    private static UUID taxonNodeUuid1;
    private static UUID taxonNodeUuid2;
    private static UUID taxonNodeUuid3;
    private static UUID taxonNodeUuid4;
    private static UUID taxonNodeUuid5;


    private static Set<TaxonNode> taxonNodes;
    private static Set<CdmEntityIdentifier> ceis;

    @BeforeClass
    public static void initializeUpdatedObjects() {
        taxonNodeUuid1 = UUID.fromString("b85b5b78-6760-409f-ac91-bb89e95ff2a1");
        taxonNodeUuid2 = UUID.fromString("6ad8e9e2-f5f6-41ad-aa30-f62a903650db");
        taxonNodeUuid3 = UUID.fromString("91698cec-615f-4472-9002-feda1a6acded");
        taxonNodeUuid4 = UUID.fromString("b8439f51-6b96-445a-b401-7a836ba1cf58");
        taxonNodeUuid5 = UUID.fromString("629b3fd3f-29b4-4011-ab12-9c1ad1607dbd");

    }

    @Test
    public void testUpdateResultWithCdmEntities() throws ExecutionException {
        TaxonNode taxonNode1 = (TaxonNode) ProxyUtils.deproxy(taxonNodeService.load(taxonNodeUuid1));
        TaxonNode taxonNode2 = (TaxonNode) ProxyUtils.deproxy(taxonNodeService.load(taxonNodeUuid2));
        TaxonNode taxonNode3 = (TaxonNode) ProxyUtils.deproxy(taxonNodeService.load(taxonNodeUuid3));
        TaxonNode taxonNode4 = (TaxonNode) ProxyUtils.deproxy(taxonNodeService.load(taxonNodeUuid4));
        TaxonNode taxonNode5 = (TaxonNode) ProxyUtils.deproxy(taxonNodeService.load(taxonNodeUuid5));

        taxonNodes = new HashSet<TaxonNode>();
        taxonNodes.add(taxonNode1);
        taxonNodes.add(taxonNode2);
        taxonNodes.add(taxonNode3);
        taxonNodes.add(taxonNode4);
        taxonNodes.add(taxonNode5);

        UpdateResult result = new UpdateResult();
        result.addUpdatedObjects(taxonNodes);
        result.setStatus(Status.OK);

        sessionOwner.setExpectedUpdatedObjects(taxonNodes);

        operation = new RemotingTestUpdateOperation(result, sessionOwner, false);
        operation.execute(monitor, info);

        Assert.assertEquals(taxonNodes, sessionOwner.getEventUpdatedObjects());
    }

    @Test
    public void testUpdateResultWithCdmIdentities() throws ExecutionException {
        TaxonNode taxonNode1 = (TaxonNode) ProxyUtils.deproxy(taxonNodeService.load(taxonNodeUuid1));
        TaxonNode taxonNode2 = (TaxonNode) ProxyUtils.deproxy(taxonNodeService.load(taxonNodeUuid2));
        TaxonNode taxonNode3 = (TaxonNode) ProxyUtils.deproxy(taxonNodeService.load(taxonNodeUuid3));
        TaxonNode taxonNode4 = (TaxonNode) ProxyUtils.deproxy(taxonNodeService.load(taxonNodeUuid4));
        TaxonNode taxonNode5 = (TaxonNode) ProxyUtils.deproxy(taxonNodeService.load(taxonNodeUuid5));

        taxonNodes = new HashSet<TaxonNode>();
        taxonNodes.add(taxonNode1);
        taxonNodes.add(taxonNode2);
        taxonNodes.add(taxonNode3);
        taxonNodes.add(taxonNode4);
        taxonNodes.add(taxonNode5);

        ceis = new HashSet<CdmEntityIdentifier>();
        ceis.add(new CdmEntityIdentifier(taxonNode3.getId(),TaxonNode.class));
        ceis.add(new CdmEntityIdentifier(taxonNode4.getId(),TaxonNode.class));
        ceis.add(new CdmEntityIdentifier(taxonNode5.getId(),TaxonNode.class));

        UpdateResult result = new UpdateResult();
        result.addUpdatedObject(taxonNode1);
        result.addUpdatedObject(taxonNode2);
        result.addUpdatedCdmIds(ceis);

        result.setStatus(Status.OK);
        sessionOwner.setExpectedUpdatedObjects(taxonNodes);
        operation = new RemotingTestUpdateOperation(result, sessionOwner, false);
        operation.execute(monitor, info);

        Assert.assertEquals(taxonNodes, sessionOwner.getEventUpdatedObjects());
    }

    @Test
    public void testUpdateResultWithLoadedCdmIdentities() throws ExecutionException {
        TaxonNode taxonNode1 = (TaxonNode) ProxyUtils.deproxy(taxonNodeService.load(taxonNodeUuid1));
        TaxonNode taxonNode2 = (TaxonNode) ProxyUtils.deproxy(taxonNodeService.load(taxonNodeUuid2));
        TaxonNode taxonNode3 = (TaxonNode) ProxyUtils.deproxy(taxonNodeService.load(taxonNodeUuid3));
        TaxonNode taxonNode4 = (TaxonNode) ProxyUtils.deproxy(taxonNodeService.load(taxonNodeUuid4));
        TaxonNode taxonNode5 = (TaxonNode) ProxyUtils.deproxy(taxonNodeService.load(taxonNodeUuid5));

        taxonNodes = new HashSet<TaxonNode>();
        taxonNodes.add(taxonNode1);
        taxonNodes.add(taxonNode2);
        taxonNodes.add(taxonNode3);
        taxonNodes.add(taxonNode4);
        taxonNodes.add(taxonNode5);

        ceis = new HashSet<CdmEntityIdentifier>();
        ceis.add(new CdmEntityIdentifier(taxonNode3.getId(),TaxonNode.class));
        ceis.add(new CdmEntityIdentifier(taxonNode4.getId(),TaxonNode.class));
        ceis.add(new CdmEntityIdentifier(taxonNode5.getId(),TaxonNode.class));


        UpdateResult result = new UpdateResult();
        result.addUpdatedObject(taxonNode1);
        result.addUpdatedObject(taxonNode2);
        result.addUpdatedCdmIds(ceis);
        result.addUpdatedCdmId(new CdmEntityIdentifier(46,TaxonNode.class));

        result.setStatus(Status.OK);
        sessionOwner.setExpectedUpdatedObjects(taxonNodes);
        operation = new RemotingTestUpdateOperation(result, sessionOwner, false);
        operation.execute(monitor, info);

        Assert.assertEquals(taxonNodes, sessionOwner.getEventUpdatedObjects());
    }

    @Test
    public void testDataChange() throws ExecutionException {
        TaxonNode taxonNode = (TaxonNode) ProxyUtils.deproxy(taxonNodeService.load(taxonNodeUuid1));
        CdmEntityIdentifier cei = new CdmEntityIdentifier(taxonNode.getId(), TaxonNode.class);

        taxonNodes = new HashSet<TaxonNode>();
        taxonNodes.add(taxonNode);

        int taxonNodeCountChildren = taxonNode.getCountChildren();
        sessionOwner.setExpectedUpdatedObjects(taxonNodes);

        operation = new RemotingTestUpdateOperation(cei, sessionOwner, false);
        operation.execute(monitor, info);

        Set eventUpdatedObjects = sessionOwner.getEventUpdatedObjects();
        Assert.assertEquals(taxonNodes, eventUpdatedObjects);
        Assert.assertEquals(taxonNodeCountChildren + 1, ((TaxonNode)eventUpdatedObjects.iterator().next()).getCountChildren());
    }

}
