// $Id$
/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.service;

import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashSet;
import java.util.List;
import java.util.UUID;

import org.junit.Assert;
import org.junit.Test;
import org.unitils.dbunit.annotation.DataSet;

import eu.etaxonomy.cdm.api.application.CdmApplicationState;
import eu.etaxonomy.cdm.io.common.ExportResult;
import eu.etaxonomy.cdm.io.csv.caryophyllales.out.CsvNameExportConfigurator;
import eu.etaxonomy.cdm.io.csv.redlist.demo.CsvDemoExportConfigurator;
import eu.etaxonomy.cdm.io.jaxb.JaxbExportConfigurator;
import eu.etaxonomy.cdm.io.sdd.out.SDDExportConfigurator;
import eu.etaxonomy.cdm.io.service.IIOService;
import eu.etaxonomy.taxeditor.httpinvoker.BaseRemotingTest;

/**
 * @author cmathew
 * @date 31 Jul 2015
 *
 */
@DataSet
public class IOServiceExportTest extends BaseRemotingTest {

    private final IIOService ioService = CdmApplicationState.getIOService();
    
    @Test
    public void exportJaxbTest() throws IOException {
        JaxbExportConfigurator config = JaxbExportConfigurator.NewInstance(null, null);
        ExportResult result = ioService.export(config);
        List<byte[]> dataList = result.getExportData();
        Assert.assertEquals(1, dataList.size());
        byte[] exportData = dataList.get(0);
        String savePath = "target/exportjaxb.xml";
        FileOutputStream stream = null;

        stream = new FileOutputStream(savePath);
        stream.write(exportData);
        stream.close();
    }

    @Test
    public void exportSDDTest() throws IOException {
        SDDExportConfigurator config = SDDExportConfigurator.NewInstance(null, null, null);
        ExportResult result = ioService.export(config);
        List<byte[]> dataList = result.getExportData();
        Assert.assertEquals(1, dataList.size());
        byte[] exportData = dataList.get(0);
        String savePath = "target/exportsdd.xml";
        FileOutputStream stream = null;

        stream = new FileOutputStream(savePath);
        stream.write(exportData);
        stream.close();

    }

    @Test
    public void exportCSVTest() throws IOException {
        HashSet<UUID> set = new HashSet<UUID>();
        set.add(UUID.fromString("0c2b5d25-7b15-4401-8b51-dd4be0ee5cab"));
        CsvDemoExportConfigurator config = CsvDemoExportConfigurator.NewInstance(null, null);
        config.setClassificationUuids(set);
        config.setDoTaxa(true);
        config.setHasHeaderLines(true);
        config.setClassification(true);
        config.setTaxonName(true);
        config.setAuthor(true);
        config.setRank(true);
        config.setLastChange(true);

        ExportResult result = ioService.export(config);
        List<byte[]> dataList = result.getExportData();
        Assert.assertEquals(1, dataList.size());
        byte[] exportData = dataList.get(0);
        String savePath = "target/exportcsv.xml";
        saveToFile(savePath, exportData);
    }

    @Test
    public void exportCSVNameTest() throws IOException {
        CsvNameExportConfigurator config = CsvNameExportConfigurator.NewInstance(null,null);
        ExportResult result = ioService.export(config);
        List<byte[]> dataList = result.getExportData();
        Assert.assertEquals(1, dataList.size());
        byte[] exportData = dataList.get(0);
        String savePath = "target/exportcsvname.xml";
        saveToFile(savePath, exportData);
    }

    @Test
    public void exportCSVPrintTest() throws IOException {
        CsvNameExportConfigurator config = CsvNameExportConfigurator.NewInstance(null,null);
        config.setNamesOnly(false);
        config.setClassificationUUID(UUID.fromString("0c2b5d25-7b15-4401-8b51-dd4be0ee5cab"));
        ExportResult result = ioService.export(config);
        List<byte[]> dataList = result.getExportData();
        Assert.assertEquals(1, dataList.size());
        byte[] exportData = dataList.get(0);
        String savePath = "target/exportcsvprint.xml";
        saveToFile(savePath, exportData);
    }

    private static void saveToFile(String filePath, byte[] data) throws IOException {
        FileOutputStream stream = null;
        stream = new FileOutputStream(filePath);
        stream.write(data);
        stream.close();
    }
}
