// $Id$
/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.service;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;

import org.junit.Assert;
import org.junit.Ignore;
import org.junit.Test;
import org.unitils.dbunit.annotation.DataSet;

import eu.etaxonomy.cdm.api.application.CdmApplicationState;
import eu.etaxonomy.cdm.api.service.INameService;
import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.database.DbSchemaValidation;
import eu.etaxonomy.cdm.io.common.IImportConfigurator.SOURCE_TYPE;
import eu.etaxonomy.cdm.io.common.ImportResult;
import eu.etaxonomy.cdm.io.excel.taxa.NormalExplicitImportConfigurator;
import eu.etaxonomy.cdm.io.sdd.in.SDDImportConfigurator;
import eu.etaxonomy.cdm.io.service.IIOService;
import eu.etaxonomy.cdm.io.specimen.abcd206.in.Abcd206ImportConfigurator;
import eu.etaxonomy.cdm.io.specimen.excel.in.SpecimenCdmExcelImportConfigurator;
import eu.etaxonomy.cdm.io.tcsxml.in.TcsXmlImportConfigurator;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.taxeditor.httpinvoker.BaseRemotingTest;

/**
 *
 * Note : One issue with this test class is that when the tests are run a second time
 * against the same server there are errors due to the fact that not all the imported data is deleted.
 * This needs to be fixed
 * @author cmathew
 * @date 5 Aug 2015
 *
 */
@Ignore // seems to cause problems with other tests due to data not deleted
@DataSet
public class IOServiceImportTest extends BaseRemotingTest {

    private final IIOService ioService = CdmApplicationState.getIOService();
    private final INameService nameService = CdmApplicationState.getCurrentAppConfig().getNameService();
    private final IOccurrenceService occurrenceService = CdmApplicationState.getCurrentAppConfig().getOccurrenceService();

    @Test
    public void importNormalExplicit() throws IOException {
        String importFilePath = "src/test/resources/imports/NormalExplicitImport.xls";
        NormalExplicitImportConfigurator config =
                NormalExplicitImportConfigurator.NewInstance(null, null, null, null);
        config.setNomenclaturalCode(NomenclaturalCode.ICNB);
        config.setDbSchemaValidation(DbSchemaValidation.CREATE);
        ImportResult result = ioService.importData(config, fileToByteArray(importFilePath), SOURCE_TYPE.URI);
        Assert.assertTrue(result.isSuccess());
        List<TaxonNameBase> list = nameService.list(TaxonNameBase.class, null, null, null, null);
        Assert.assertEquals(9,list.size());
    }


    @Test
    public void importAbcd() throws IOException {
        String importFilePath = "src/test/resources/imports/ABCDImport.xml";
        Abcd206ImportConfigurator config =
                Abcd206ImportConfigurator.NewInstance(null, null);
        config.setNomenclaturalCode(NomenclaturalCode.ICNB);
        config.setDbSchemaValidation(DbSchemaValidation.CREATE);
        ImportResult result = ioService.importData(config, fileToByteArray(importFilePath), SOURCE_TYPE.INPUTSTREAM);
        Assert.assertTrue(result.isSuccess());
        Assert.assertTrue(!result.getReports().isEmpty());
        saveToFile("target/importAbcdReport.txt", result.getReports().get(0));
        List<TaxonNameBase> list = nameService.list(TaxonNameBase.class, null, null, null, null);
        Assert.assertEquals(2,list.size());
    }

    @Test
    public void importSDD() throws IOException {
        String importFilePath = "src/test/resources/imports/SDD.xml";
        SDDImportConfigurator config =
                SDDImportConfigurator.NewInstance(null, null);
        config.setNomenclaturalCode(NomenclaturalCode.ICNB);
        config.setDbSchemaValidation(DbSchemaValidation.CREATE);
        ImportResult result = ioService.importData(config, fileToByteArray(importFilePath), SOURCE_TYPE.URI);
        Assert.assertTrue(result.isSuccess());
        List<TaxonNameBase> list = nameService.list(TaxonNameBase.class, null, null, null, null);
        Assert.assertEquals(9,list.size());
    }

    @Test
    public void importSpecimenExcel() throws IOException {
        String importFilePath = "src/test/resources/imports/SpecimenExcelImport.xls";
        SpecimenCdmExcelImportConfigurator config = SpecimenCdmExcelImportConfigurator.NewInstance(null, null);
        config.setNomenclaturalCode(NomenclaturalCode.ICNB);
        config.setDbSchemaValidation(DbSchemaValidation.CREATE);
        ImportResult result = ioService.importData(config, fileToByteArray(importFilePath), SOURCE_TYPE.URI);
        Assert.assertTrue(result.isSuccess());
//        List<SpecimenOrObservationBase> occurrences = occurrenceService.list(SpecimenOrObservationBase.class, null, null, null, null);
//        Assert.assertEquals(3,occurrences.size());
    }

    @Test
    public void importTCS() throws IOException {
        String importFilePath = "src/test/resources/imports/TcsImport.xml";
        TcsXmlImportConfigurator config = TcsXmlImportConfigurator.NewInstance(null, null);
        config.setNomenclaturalCode(NomenclaturalCode.ICNB);
        config.setDbSchemaValidation(DbSchemaValidation.CREATE);
        ImportResult result = ioService.importData(config, fileToByteArray(importFilePath), SOURCE_TYPE.URI);
        Assert.assertTrue(result.isSuccess());
        List<TaxonNameBase> list = nameService.list(TaxonNameBase.class, null, null, null, null);
        Assert.assertEquals(16,list.size());
    }

    private byte[] fileToByteArray(String filePath) throws IOException {
        Path path = Paths.get(filePath);
        return Files.readAllBytes(path);
    }

    private static void saveToFile(String filePath, byte[] data) {
        FileOutputStream stream = null;
        try {
            stream = new FileOutputStream(filePath);
            stream.write(data);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if(stream != null) {
                    stream.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

}
