package eu.etaxonomy.taxeditor.session;

import java.util.Iterator;
import java.util.List;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.unitils.dbunit.annotation.DataSet;

import eu.etaxonomy.cdm.api.service.IPolytomousKeyNodeService;
import eu.etaxonomy.cdm.api.service.IPolytomousKeyService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.LanguageString;
import eu.etaxonomy.cdm.model.description.KeyStatement;
import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.cdm.persistence.dao.description.IPolytomousKeyNodeDao;
import eu.etaxonomy.cdm.persistence.dto.MergeResult;
import eu.etaxonomy.taxeditor.httpinvoker.RemotingSessionAwareTest;

//FIXME:Remoting fix data issue : User#30
//@Ignore
@DataSet
public class CdmEntitySessionManagerTest extends RemotingSessionAwareTest {

	@SuppressWarnings("unused")
	private static final Logger logger = Logger.getLogger(CdmEntitySessionManagerTest.class);


	IPolytomousKeyService polytomousKeyService = getRemoteApplicationController().getPolytomousKeyService();
	IPolytomousKeyNodeService polytomousKeyNodeService = getRemoteApplicationController().getPolytomousKeyNodeService();

	private final UUID polytomousKeyUuid = UUID.fromString("0d53ba20-7de4-4baa-bd8a-401048447d66");
	private final Language english = Language.getLanguageFromUuid(Language.uuidEnglish);


	@BeforeClass
	public static void initializeCdmEntitySessionManagerTest() {
	}


	@Test
	public void manageNullSessionTest() {

		PolytomousKey pKey = CdmBase.deproxy(polytomousKeyService.find(polytomousKeyUuid),PolytomousKey.class);
		cdmEntitySessionManager.bind(null);
		Assert.assertSame(pKey, pKey);
		Assert.assertNull(getActiveSession());
	}

	@Test
	public void manageSessionWithObjectTest() {
		PolytomousKey pKey = CdmBase.deproxy(polytomousKeyService.find(polytomousKeyUuid),PolytomousKey.class);

		Assert.assertNotNull(getActiveSession());


		// FIXME:Remoting fix test of setting root entities
		//Assert.assertEquals(rootEntities.size(),1);
		//Assert.assertSame(rootEntities.get(0), pKey);

		String upTitleCache = "Updated Title Cache";
		String upStatement = "Updated Statement";
		pKey.setTitleCache(upTitleCache, true);

		PolytomousKeyNode node = pKey.getRoot().getChildAt(0);
		node.getStatement().getLabel(english).setText(upStatement);
		polytomousKeyService.merge(pKey, true);

		pKey = CdmBase.deproxy(polytomousKeyService.find(polytomousKeyUuid),PolytomousKey.class);


		Assert.assertEquals(pKey.getTitleCache(), upTitleCache);
		node = pKey.getRoot().getChildAt(0);
		KeyStatement statement = node.getStatement();
		LanguageString label = statement.getLabel(english);
		Assert.assertEquals(label.getText(), upStatement);
	}

	@Test
	public void manageSessionWithListTest() {
		List<PolytomousKey> pKeys = polytomousKeyService.list(PolytomousKey.class, null, null, null, null);
		Assert.assertNotNull(getActiveSession());


		// FIXME:Remoting fix test of setting root entities
		//Assert.assertEquals(rootEntities.size(),2);
		//Assert.assertSame(rootEntities.get(0),pKeys.get(0));
		//Assert.assertEquals(rootEntities.get(0).getUuid(),UUID.fromString("9d8bf4f6-a70a-4b80-8556-2ccfb436ff01"));
		//Assert.assertSame(rootEntities.get(1),pKeys.get(1));
		//Assert.assertEquals(rootEntities.get(1).getUuid(),UUID.fromString("0d53ba20-7de4-4baa-bd8a-401048447d66"));

		String upTitleCache = "Updated Title Cache";
		String upStatement = "Updated Statement";
		pKeys.get(0).setTitleCache(upTitleCache, true);
		polytomousKeyService.merge(pKeys.get(0));
		PolytomousKey key = pKeys.get(1);
		PolytomousKeyNode root = key.getRoot();
		root = HibernateProxyHelper.deproxy(root, PolytomousKeyNode.class);
		PolytomousKeyNode node = root.getChildAt(0);
		List<PolytomousKeyNode> nodes = root.getChildren();
		node.getStatement().getLabel(english).setText(upStatement);
		
		MergeResult<PolytomousKey> result = polytomousKeyService.merge(key, true);
		key = result.getMergedEntity();
		pKeys = polytomousKeyService.list(PolytomousKey.class, null, null, null, null);

		Assert.assertEquals(pKeys.get(0).getTitleCache(), upTitleCache);
		key = pKeys.get(1);
		root = key.getRoot();
		nodes = root.getChildren();
		Iterator<PolytomousKeyNode> it = nodes.iterator();
		if (it.hasNext()){
			node = it.next();
		} else {
			Assert.fail();
		}
		Assert.assertEquals(node.getStatement().getLabel(english).getText(), upStatement);

	}

}
