package eu.etaxonomy.taxeditor.test;

import java.util.List;

import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.swtbot.eclipse.finder.SWTWorkbenchBot;
import org.eclipse.swtbot.eclipse.finder.matchers.WidgetMatcherFactory;
import org.eclipse.swtbot.eclipse.finder.waits.Conditions;
import org.eclipse.swtbot.eclipse.finder.widgets.SWTBotView;
import org.eclipse.swtbot.swt.finder.exceptions.WidgetNotFoundException;
import org.eclipse.swtbot.swt.finder.finders.ChildrenControlFinder;
import org.eclipse.swtbot.swt.finder.widgets.SWTBotButton;
import org.eclipse.swtbot.swt.finder.widgets.SWTBotTree;

public class SwtBotUtils {

	private SWTWorkbenchBot bot;

	public SwtBotUtils(SWTWorkbenchBot bot) {
		this.bot = bot;
	}

	/**
	 * @return
	 * @throws WidgetNotFoundException
	 */
	public SWTBotTree tree(Widget widget) throws WidgetNotFoundException {
		List<Tree> controls = new ChildrenControlFinder(widget).findControls(WidgetMatcherFactory.widgetOfType(Tree.class));
		if (controls.isEmpty()) {
            throw new WidgetNotFoundException("Could not find any tree");
        }
		SWTBotTree tree = new SWTBotTree(controls.get(0));
		return tree;
	}

	/**
	 *
	 * @param index the index of the SelectionElement
	 * @param dialogName the title of the expected dialog
	 */
	public void openFilteredSelection(int index, String dialogName){
		SWTBotButton existing = bot.buttonWithTooltip("Browse existing", index);
		existing.click();
		bot.waitUntil(Conditions.shellIsActive(dialogName));
	}

	/**
	 * @return
	 * @throws WidgetNotFoundException
	 */
	public SWTBotView view(String title) throws WidgetNotFoundException {
		return getBot().viewByTitle(title);
	}

	public void cancel() {
		clickButton("Cancel");
	}

	public void ok() {
		clickButton("OK");
	}

	public void finish() {
		clickButton("Finish");
	}

	public void clickButton(String label) {
		getBot().button(label).click();
	}

	public String getActiveShellHeadline(){
		return getBot().activeShell().getText();
	}

	public SWTWorkbenchBot getBot() {
		return bot;
	}

	public void setBot(SWTWorkbenchBot bot) {
		this.bot = bot;
	}
}
