// $Id$
/**
 * Copyright (C) 2015 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.dialogs;

import java.util.List;

import org.junit.Assert;
import org.junit.Ignore;
import org.junit.Test;
import org.unitils.UnitilsJUnit4;

import eu.etaxonomy.cdm.api.application.CdmApplicationState;
import eu.etaxonomy.cdm.config.CdmSourceException;
import eu.etaxonomy.taxeditor.remoting.server.CDMServerException;
import eu.etaxonomy.taxeditor.remoting.source.CdmServerInfo;
import eu.etaxonomy.taxeditor.remoting.source.CdmServerInfo.CdmInstanceInfo;
import eu.etaxonomy.taxeditor.remoting.source.CdmServerInfoConfig;

/**
 * @author cmathew
 * @date 26 Jan 2015
 *
 */

public class CdmServerInfoTest extends UnitilsJUnit4 {

    @Test
    public void pingServersTest() {
        List<CdmServerInfo> servers = CdmServerInfo.getCdmServers();
        for(CdmServerInfo server : servers) {
            server.pingServer();
        }
    }


    @Ignore // this should be targetting integration or production
    @Test
    public void pingInstancesTest() {
        CdmServerInfo csii = new CdmServerInfo(new CdmServerInfoConfig("edit-test", "test.e-taxonomy.eu", 8080, "cdmserver/", false));
        try {
            csii.refreshInstances();
            List<CdmInstanceInfo> instances = csii.getInstances();
            for(CdmInstanceInfo instance : instances) {
                Assert.assertTrue(csii.pingInstance(instance, 8080));
            }
        } catch (CDMServerException e) {
            Assert.fail("Execption should not be thrown here");
        }
    }

    @Test
    public void refreshWrongInstancesTest() {
        CdmServerInfo wrongCsii = new CdmServerInfo(new CdmServerInfoConfig("local", "local", 8080, "noserver", false));
        try {
            wrongCsii.refreshInstances();
            List<CdmInstanceInfo> instances = wrongCsii.getInstances();
            Assert.fail("Execption should be thrown here");
        } catch (CDMServerException e) {

        }
    }

    @Ignore // this should be targetting integration or production
    @Test
    public void refreshInstancesTest() throws CDMServerException {
        CdmServerInfo cdmServerInfo = new CdmServerInfo(new CdmServerInfoConfig("edit-test", "test.e-taxonomy.eu", 8080, "cdmserver/", false));
        cdmServerInfo.refreshInstances();
        List<CdmInstanceInfo> instances = cdmServerInfo.getInstances();
        Assert.assertTrue(instances != null && !instances.isEmpty());
    }

    @Test
    public void compareCdmlibServicesVersionTest() throws CdmSourceException {
        String editorVersion = CdmApplicationState.getCdmlibVersion();
        String editorCdmlibLastModified = CdmApplicationState.getCdmlibLastModified();
        Long editorCdmlibLastModifiedLong = Long.valueOf(editorCdmlibLastModified);

        String[] editorVersionSplit = editorVersion.split("\\.");

        Assert.assertEquals(0, CdmServerInfo.compareCdmlibServicesVersion(editorVersion, editorCdmlibLastModified));


        int editorVersionMajor = Integer.valueOf(editorVersionSplit[0]);
        int editorVersionMinor = Integer.valueOf(editorVersionSplit[1]);
        int editorVersionPatch = Integer.valueOf(editorVersionSplit[2]);

        String serverVersionMajorNew = String.valueOf(editorVersionMajor+1) + "." + String.valueOf(editorVersionMinor) + "." +  String.valueOf(editorVersionPatch);
        Assert.assertTrue(CdmServerInfo.compareCdmlibServicesVersion(serverVersionMajorNew, editorCdmlibLastModified) > 0);

        String serverVersionMinorNew = String.valueOf(editorVersionMajor) + "." + String.valueOf(editorVersionMinor+1) + "." + String.valueOf(editorVersionPatch);
        Assert.assertTrue(CdmServerInfo.compareCdmlibServicesVersion(serverVersionMinorNew, editorCdmlibLastModified) > 0);

        String serverVersionPatchNew = String.valueOf(editorVersionMajor) + "." + String.valueOf(editorVersionMinor) + "." + String.valueOf(editorVersionPatch+1);
        Assert.assertTrue(CdmServerInfo.compareCdmlibServicesVersion(serverVersionPatchNew, editorCdmlibLastModified) > 0);

        String serverVersionMajorOld = String.valueOf(editorVersionMajor-1) + "." + String.valueOf(editorVersionMinor) + "." + String.valueOf(editorVersionPatch);
        Assert.assertTrue(CdmServerInfo.compareCdmlibServicesVersion(serverVersionMajorOld, editorCdmlibLastModified) < 0);

        String serverVersionMinorOld = String.valueOf(editorVersionMajor) + "." + String.valueOf(editorVersionMinor-1) + "." + String.valueOf(editorVersionPatch);
        Assert.assertTrue(CdmServerInfo.compareCdmlibServicesVersion(serverVersionMinorOld, editorCdmlibLastModified) < 0);

        String serverVersionPatchOld = String.valueOf(editorVersionMajor) + "." + String.valueOf(editorVersionMinor) + "." + String.valueOf(editorVersionPatch-1);
        Assert.assertTrue(CdmServerInfo.compareCdmlibServicesVersion(serverVersionPatchOld, editorCdmlibLastModified) < 0);

        String serverCdmlibLastModifiedNew = String.valueOf(editorCdmlibLastModifiedLong+1);
        Assert.assertTrue(CdmServerInfo.compareCdmlibServicesVersion(editorVersion, serverCdmlibLastModifiedNew) > 0);

        String serverCdmlibLastModifiedOld = String.valueOf(editorCdmlibLastModifiedLong-1);
        Assert.assertTrue(CdmServerInfo.compareCdmlibServicesVersion(editorVersion, serverCdmlibLastModifiedOld) < 0);

    }

    @Test
    public void convertToServerConfigTest() {


    }
}
